<?php

define("extproc_stream_filter_PREFIX", "extproc.");

class extproc_stream_filter extends php_user_filter
{
    private $proc_in;
    private $proc_out;
    private $proc;
    private $last_bucket;
    
    public function onCreate()
    {
        $cmd = substr($this->filtername, strlen(extproc_stream_filter_PREFIX));
    
        $this->proc = proc_open($cmd, array(0 => array('pipe', 'r'), 1 => array('pipe', 'w')), $pipes);
        if (!is_resource($this->proc))
        {
            error_log("extproc_stream_filter: proc_open failed");
            return FALSE;
        }
        
        $this->proc_in = $pipes[0];
        $this->proc_out = $pipes[1];
        
        stream_set_blocking($this->proc_in, FALSE);
        stream_set_blocking($this->proc_out, FALSE);

        return TRUE;
    }

    public function filter($in, $out, &$consumed, $closing)
    {
        $consumed = 0;
        $rdata = "";

        while ($bucket = stream_bucket_make_writeable($in))
        {
            $consumed += $bucket->datalen;

            $wdata = $bucket->data;
            while ($wdata)
            {
                $wsize = fwrite($this->proc_in, $wdata);
                if ($wsize === FALSE)
                {
                    error_log("extproc_stream_filter: fwrite failed");
                    return PSFS_ERR_FATAL;
                }
                if (!fflush($this->proc_in))
                {
                    error_log("extproc_stream_filter: fflush failed");
                    return PSFS_ERR_FATAL;
                }
                
                $wdata = substr($wdata, $wsize);

                $rsize = 0;
                while($chunk = fread($this->proc_out, 65536))
                {
                    $rdata .= $chunk;
                    $rsize += strlen($chunk);
                }
                if ($chunk === FALSE)
                {
                    error_log("extproc_stream_filter: fread failed");
                    return PSFS_ERR_FATAL;
                }
                
                if (!$wsize && !$rsize)
                {
                    $status = proc_get_status($this->proc);
                    if (!$status || !$status["running"])
                    {
                        error_log("extproc_stream_filter: proc_get_status reports failure");
                        return PSFS_ERR_FATAL;
                    }
                    
                    $read = array($this->proc_out);
                    $write = array($this->proc_in);
                    $except = NULL;
                    if (!stream_select($read, $write, $except, 3))
                    {
                        error_log("extproc_stream_filter: io timeout");
                        return PSFS_ERR_FATAL;
                    }
                }
            }

            $this->last_bucket = $bucket;
        }
        
        $bucket = $this->last_bucket;

        if ($closing)
        {
            fclose($this->proc_in);
            stream_set_blocking($this->proc_out, TRUE);

            while($chunk = fread($this->proc_out, 65536))
            {
                $rdata .= $chunk;
            }
        }

        if ($rdata)
        {
            $bucket->data = $rdata;
            $bucket->datalen = strlen($rdata);
            stream_bucket_append($out, $bucket);
            return PSFS_PASS_ON;
        }

        return PSFS_FEED_ME;
    }

    public function onClose()
    {
        proc_close($this->proc);
    }
}

if (!stream_filter_register(extproc_stream_filter_PREFIX . '*', "extproc_stream_filter"))
    exit("Failed to register extproc_stream_filter");

?>
