<?php

include_once "inc/config.php";

function add_playlist($name)
{
    global $pdo;
    static $playlist_name_id_map = array();

    if(isset($playlist_name_id_map[$name]))
        return $playlist_name_id_map[$name];

    $pdo->exec("insert into playlists (name) values (" . $pdo->quote($name) . ")");
    $playlist_id = $pdo->lastInsertId('id');
    $playlist_name_id_map[$name] = $playlist_id;

    return $playlist_id;
}

function add_channel_category($name)
{
    global $pdo;
    static $channel_category_name_id_map = array();

    if (isset($channel_category_name_id_map[$name]))
        return $channel_category_name_id_map[$name];

    if ($name)
    {
        $pdo->exec("insert into channel_categories (name) values (" . $pdo->quote($name) . ")");
        $category_id = $pdo->lastInsertId('id');
    }
    else
    {
        $category_id = 0;
        $pdo->exec("insert into channel_categories (id, name) values ($category_id, " . $pdo->quote($name) . ")");
    }

    $channel_category_name_id_map[$name] = $category_id;

    return $category_id;
}

function add_channel_icon($url)
{
    static $name = 0;
    if (isset($url) && $url)
    {
        ++$name;
        $ext = strrchr($url, '.');
        if (copy($url, data_ICON_DIR . "/" . data_ICON_TMP_PREFIX . "$name$ext"))
            return data_ICON_DIR . "/" . "$name$ext";
    }
    return "";
}

function init_channel_icon_dir()
{
    $files = glob(data_ICON_DIR . "/" . data_ICON_TMP_PREFIX . "*");
    foreach ($files as $file)
        unlink($file);
}

function commit_channel_icon_dir()
{
    $files = glob(data_ICON_DIR . "/*");
    foreach ($files as $file)
        if (substr(substr(strrchr($file, '/'), 1), 0, strlen(data_ICON_TMP_PREFIX)) != data_ICON_TMP_PREFIX)
            unlink($file);
    foreach ($files as $file)
        if (substr(substr(strrchr($file, '/'), 1), 0, strlen(data_ICON_TMP_PREFIX)) == data_ICON_TMP_PREFIX)
            rename($file, data_ICON_DIR . "/" . substr(substr(strrchr($file, '/'), 1), strlen(data_ICON_TMP_PREFIX)));
}

function channel_name_mangled($name)
{
    global $settings;

    if ($settings["mangling"])
    {
        $name = strtolower($name);
        $name = str_replace(" ", "_", $name);
        $name = str_replace(":", "_", $name);
    }

    return $name;
}

$channel_name_idset_map = array();

function add_channel($names, $description, $url, $icon, $category_id)
{
    global $pdo;
    static $channel_url_id_map = array();
    global $channel_name_idset_map;
    global $channel_aliases;

    if (isset($channel_url_id_map[$url]))
    {
        $channel_id = $channel_url_id_map[$url];
    }

    if (isset($channel_id))
    {
        if ($icon)
            $pdo->exec("update channels set icon = " . $pdo->quote($icon) . " where id = $channel_id");
        if ($category_id)
            $pdo->exec("update channels set category_id = $category_id where id = $channel_id");
    }
    else
    {
        $pdo->exec("insert into channels (name, description, url, icon, category_id) values (" . $pdo->quote($names[0]) . ", " . $pdo->quote($description) . ", " . $pdo->quote($url) . ", " . $pdo->quote($icon) . ", $category_id)");
        $channel_id = $pdo->lastInsertId('id');
        $channel_url_id_map[$url] = $channel_id;
    }

    foreach ($names as $name)
    {
        $channel_name_idset_map[$name][$channel_id] = 1;
        $channel_name_idset_map[channel_name_mangled($name)][$channel_id] = 1;
        if (isset($channel_aliases[$name]))
            foreach ($channel_aliases[$name] as $alias)
            {
                $channel_name_idset_map[$alias][$channel_id] = 1;
                $channel_name_idset_map[channel_name_mangled($alias)][$channel_id] = 1;
            }
    }

    return $channel_id;
}

function update_channel_icon($channel_id, $icon)
{
    global $pdo;

    foreach ($pdo->query("select icon from channels where id = $channel_id") as $channel_row)
    {
        if (isset($channel_row["icon"]))
        {
            if (!$channel_row["icon"])
                $pdo->exec("update channels set icon = " . $pdo->quote($icon) . " where id = $channel_id");
        }
    }
}

function add_programme_category($name)
{
    global $pdo;
    static $programme_category_name_id_map = array();

    if (isset($programme_category_name_id_map[$name]))
        return $programme_category_name_id_map[$name];

    if ($name)
    {
        $pdo->exec("insert into programme_categories (name) values (" . $pdo->quote($name) . ")");
        $category_id = $pdo->lastInsertId('id');
    }
    else
    {
        $category_id = 0;
        $pdo->exec("insert into programme_categories (id, name) values ($category_id, " . $pdo->quote($name) . ")");
    }

    $programme_category_name_id_map[$name] = $category_id;

    return $category_id;
}

function add_programme($name, $description, $category_id)
{
    global $pdo;
    static $programme_name_description_category_id_map = array();

    if (isset($programme_name_description_category_id_map[$name][$description][$category_id]))
        return $programme_name_description_category_id_map[$name][$description][$category_id];

    $pdo->exec("insert into programmes (name, description, category_id) values (" . $pdo->quote($name) . ", " . $pdo->quote($description) . ", $category_id)");
    $programme_id = $pdo->lastInsertId('id');
    $programme_name_description_category_id_map[$name][$description][$category_id] = $programme_id;

    return $programme_id;
}

function add_timetable_programme($channel_id, $time_start, $time_stop, $programme_id)
{
    global $pdo;
    static $timetable_channelid_timestart_timestop_programmeid_map = array();

    if (isset($timetable_channelid_timestart_timestop_programmeid_map[$channel_id][$time_start][$time_stop][$programme_id]))
        return $timetable_channelid_timestart_timestop_programmeid_map[$channel_id][$time_start][$time_stop][$programme_id];

    $pdo->exec("insert into timetable (channel_id, time_start, time_stop, programme_id) values ($channel_id, $time_start, $time_stop, $programme_id)");
    $timetable_id = $pdo->lastInsertId('id');
    $timetable_channelid_timestart_timestop_programmeid_map[$channel_id][$time_start][$time_stop][$programme_id] = $timetable_id;

    return $timetable_id;
}

function add_xmltv($name)
{
    global $pdo;
    static $xmltv_name_id_map = array();

    if (isset($xmltv_name_id_map[$name]))
        return $xmltv_name_id_map[$name];

    $pdo->exec("insert into xmltvs (name) values (" . $pdo->quote($name) . ")");
    $xmltv_id = $pdo->lastInsertId('id');
    $xmltv_name_id_map[$name] = $xmltv_id;

    return $xmltv_id;
}

$channel_unnamed_id = 0;

function parse_m3u($name, $f)
{
    global $pdo;
    try
    {
        $playlist_id = add_playlist($name);
        #echo "Playlist $name added: id = $playlist_id\n";
        $channel_set = array();
        $channel_count = 0;
        $channel_names = array();
        $channel_category = "";
        while($line = fgets($f))
        {
            if (preg_match('/^#EXTINF:.+,\s*(.+?)\s*$/', $line, $matches))
            {
                $channel_names = array($matches[1]);
                if (preg_match('/tvg\-name="(.*?)"/', $line, $matches))
                {
                    if ($matches[1])
                        $channel_names[] = $matches[1];
                }
                if (preg_match('/tvg\-logo="(.*?)"/', $line, $matches))
                {
                    if ($matches[1])
                        $channel_names[] = $matches[1];
                }
                if (preg_match('/group\-title="(.*?)"/', $line, $matches))
                {
                    $channel_category = $matches[1];
                }
            }
            elseif (preg_match('/^\w+:\/\/\S+/', $line, $matches))
            {
                $url = $matches[0];
                if (count($channel_names) == 0)
                {
                    echo "Unnamed channel: $url\n";
                    global $channel_unnamed_id;
                    ++$channel_unnamed_id;
                    $channel_names[] = "Unnamed $channel_unnamed_id";
                }
                $channel_category_id = add_channel_category($channel_category);
                $channel_id = add_channel($channel_names, '', $url, "", $channel_category_id);
                $pdo->exec("insert into playlist_channels (playlist_id, channel_id) values($playlist_id, $channel_id)");
                #echo "Channel ", join("/", $channel_names), " added: id = $channel_id\n";
                ++$channel_count;
                foreach ($channel_names as $channel_name)
                    $channel_set[$channel_name] = 1;
                $channel_names = array();
                $channel_category = "";
            }
        }
        echo "Playlist $name:\n",
            $channel_count, " channels (", join(",", array_keys($channel_set)), ")\n";
    }
    catch (Exception $e)
    {
        echo "Exception: ", $e->getMessage(), "\n";
        return FALSE;
    }
    return TRUE;
}

define ('update_xspf_STATE_NONE', 0);
define ('update_xspf_STATE_URL', 1);
define ('update_xspf_STATE_NAME', 2);
define ('update_xspf_STATE_DESCRIPTION', 3);
define ('update_xspf_STATE_ICON', 4);

$xspf_state;
$xspf_channel_name;
$xspf_channel_url;
$xspf_channel_description;
$xspf_channel_icon;
$xspf_playlist_id;
$xspf_channel_set;
$xspf_channel_icons_count;

function xspf_start_element_handler($parser, $name, $attribs)
{
    global $xspf_state, $xspf_channel_name, $xspf_channel_url, $xspf_channel_description, $xspf_channel_icon;

    switch ($name)
    {
        case "TITLE":
            $xspf_channel_name = "";
            $xspf_state = update_xspf_STATE_NAME;
            break;
        case "LOCATION":
            $xspf_channel_url = "";
            $xspf_state = update_xspf_STATE_URL;
            break;
        case "ANNOTATION":
            $xspf_channel_description = "";
            $xspf_state = update_xspf_STATE_DESCRIPTION;
            break;
        case "IMAGE":
            $xspf_channel_icon = "";
            $xspf_state = update_xspf_STATE_ICON;
            break;
    }
}

function xspf_end_element_handler($parser, $name)
{
    global $pdo;
    global $xspf_state, $xspf_channel_name, $xspf_channel_url, $xspf_channel_description, $xspf_channel_icon,
        $xspf_playlist_id, $xspf_channel_set, $xspf_channel_icons_count;
    global $channel_unnamed_id;

    switch ($name)
    {
        case "TRACK":
            if ($xspf_channel_url)
            {
                if (!$xspf_channel_name)
                {
                    echo "Unnamed channel: $xspf_channel_url\n";
                    ++$channel_unnamed_id;
                    $xspf_channel_name = "Unnamed $channel_unnamed_id";
                }
                $channel_category_id = add_channel_category("");
                $channel_icon = add_channel_icon($xspf_channel_icon);
                if ($channel_icon)
                    ++$xspf_channel_icons_count;
                $channel_id = add_channel(array($xspf_channel_name), $xspf_channel_description, $xspf_channel_url, $channel_icon, $channel_category_id);
                $pdo->exec("insert into playlist_channels (playlist_id, channel_id) values($xspf_playlist_id, $channel_id)");
                #echo "Channel $xspf_channel_name added: id = $channel_id\n";
                $xspf_channel_set[$xspf_channel_name] = 1;
            }
            else
            {
                echo "Channel $xspf_channel_name without url\n";
            }
            $xspf_state = update_xspf_STATE_NONE;
            $xspf_channel_name = NULL;
            $xspf_channel_url = NULL;
            $xspf_channel_description = NULL;
            $xspf_channel_icon = NULL;
            break;
        case "LOCATION":
        case "TITLE":
        case "ANNOTATION":
        case "IMAGE":
            $xspf_state = update_xspf_STATE_NONE;
            break;
    }
}

function xspf_character_data_handler($parser, $data)
{
    global $xspf_state, $xspf_channel_name, $xspf_channel_url, $xspf_channel_description, $xspf_channel_icon;

    switch ($xspf_state)
    {
        case update_xspf_STATE_URL:
            $xspf_channel_url .= $data;
            break;
        case update_xspf_STATE_NAME:
            $xspf_channel_name .= $data;
            break;
        case update_xspf_STATE_DESCRIPTION:
            $xspf_channel_description .= $data;
            break;
        case update_xspf_STATE_ICON:
            $xspf_channel_icon .= $data;
            break;
    }
}

function update_xml_parser_errstr($xml_parser)
{
    return "XML parsing failed: " . xml_error_string(xml_get_error_code($xml_parser))
        . " at line:" . xml_get_current_line_number($xml_parser)
        . " column:" . xml_get_current_column_number($xml_parser)
        . " byte index:" . xml_get_current_byte_index($xml_parser);
}

function parse_xspf($name, $f)
{
    global $xspf_state, $xspf_channel_name, $xspf_channel_url, $xspf_channel_description, $xspf_channel_icon,
        $xspf_playlist_id, $xspf_channel_set, $xspf_channel_icons_count;

    $xspf_state = update_xspf_STATE_NONE;
    $xspf_channel_name = NULL;
    $xspf_channel_url = NULL;
    $xspf_channel_description = NULL;
    $xspf_channel_icon = NULL;
    $xspf_playlist_id = NULL;
    $xspf_chanels_count = 0;
    $xspf_channel_set = array();
    $xspf_channel_icons_count = 0;

    try
    {
        $xml_parser = xml_parser_create("UTF-8");
        xml_set_element_handler($xml_parser, 'xspf_start_element_handler', 'xspf_end_element_handler');
        xml_set_character_data_handler($xml_parser, 'xspf_character_data_handler');

        $playlist_id = add_playlist($name);
        #echo "Playlist $name added: id = $playlist_id\n";
        $xspf_playlist_id = $playlist_id;

        while($chunk = fread($f, 65536))
        {
            if (!xml_parse($xml_parser, $chunk))
            {
                echo update_xml_parser_errstr($xml_parser), "\n";
                return FALSE;
            }
        }
        if (!xml_parse($xml_parser, "", TRUE))
        {
            echo update_xml_parser_errstr($xml_parser), "\n";
            return FALSE;
        }

        echo "Playlist $name:\n",
            count(array_keys($xspf_channel_set)), " channels (", join(",", array_keys($xspf_channel_set)), ")\n",
            $xspf_channel_icons_count, " icons loaded\n";
    }
    catch (Exception $e)
    {
        echo "Exception: ", $e->getMessage(), "\n";
        return FALSE;
    }
    return TRUE;
}

define ('update_xmltv_STATE_NONE', 0);
define ('update_xmltv_STATE_CHANNEL_NAME', 1);
define ('update_xmltv_STATE_PROGRAMME_NAME', 2);
define ('update_xmltv_STATE_PROGRAMME_DESCRIPTION', 3);
define ('update_xmltv_STATE_PROGRAMME_CATEGORY', 4);

$xmltv_state;
$xmltv_id;
$xmltv_channel_idset_map;
$xmltv_current_channel_id;
$xmltv_current_channel_name;
$xmltv_current_channel_icon;
$xmltv_current_programme_time_start;
$xmltv_current_programme_time_stop;
$xmltv_current_programme_name;
$xmltv_current_programme_description;
$xmltv_current_programme_category;
$xmltv_unknown_channel_set; # channels not in playlists
$xmltv_channel_timetable_count;
$xmltv_timetable_unknown_count; # programs for channels not in playlists
$xmltv_channel_icons_count;

function xmltv_parse_time($xmltv_time)
{
    return strtotime($xmltv_time);
}

function xmltv_start_element_handler($parser, $name, $attribs)
{
    global $xmltv_state;
    global $xmltv_current_channel_id, $xmltv_current_channel_name, $xmltv_current_channel_icon;
    global $xmltv_current_programme_time_start, $xmltv_current_programme_time_stop, $xmltv_current_programme_name,
        $xmltv_current_programme_description, $xmltv_current_programme_category;

    switch ($name)
    {
        case "CHANNEL":
            $xmltv_current_channel_id = $attribs["ID"];
            break;
        case "DISPLAY-NAME":
            $xmltv_current_channel_name = "";
            $xmltv_state = update_xmltv_STATE_CHANNEL_NAME;
            break;
        case "PROGRAMME":
            $xmltv_current_channel_id = $attribs["CHANNEL"];
            $xmltv_current_programme_time_start = $attribs["START"];
            $xmltv_current_programme_time_stop = $attribs["STOP"];
            break;
        case "TITLE":
            $xmltv_current_programme_name = "";
            $xmltv_state = update_xmltv_STATE_PROGRAMME_NAME;
            break;
        case "DESC":
            $xmltv_current_programme_description = "";
            $xmltv_state = update_xmltv_STATE_PROGRAMME_DESCRIPTION;
            break;
        case "CATEGORY":
            $xmltv_current_programme_category = "";
            $xmltv_state = update_xmltv_STATE_PROGRAMME_CATEGORY;
            break;
        case "ICON":
            $xmltv_current_channel_icon = $attribs["SRC"];
            break;
    }
}

function xmltv_end_element_handler($parser, $name)
{
    global $pdo;
    global $channel_name_idset_map;
    global $xmltv_state, $xmltv_id, $xmltv_channel_idset_map,
        $xmltv_current_channel_id, $xmltv_current_channel_name, $xmltv_current_channel_icon,
        $xmltv_current_programme_time_start, $xmltv_current_programme_time_stop, $xmltv_current_programme_name,
        $xmltv_current_programme_description, $xmltv_current_programme_category,
        $xmltv_unknown_channel_set, $xmltv_channel_timetable_count, $xmltv_timetable_unknown_count, $xmltv_channel_icons_count,
        $xmltv_time_offset;

    switch ($name)
    {
        case "CHANNEL":
            if (isset($channel_name_idset_map[$xmltv_current_channel_name]))
                $idset_map = $channel_name_idset_map[$xmltv_current_channel_name];
            else if (isset($channel_name_idset_map[channel_name_mangled($xmltv_current_channel_name)]))
                $idset_map = $channel_name_idset_map[channel_name_mangled($xmltv_current_channel_name)];
            if (isset($idset_map))
            {
                $xmltv_channel_idset_map[$xmltv_current_channel_id] = $idset_map;
                unset($idset_map);

                $channel_icon = add_channel_icon($xmltv_current_channel_icon);
                if ($channel_icon)
                {
                    foreach (array_keys($xmltv_channel_idset_map[$xmltv_current_channel_id]) as $channel_id)
                        update_channel_icon($channel_id, $channel_icon);
                    ++$xmltv_channel_icons_count;
                }
            }
            else
            {
                #echo "Channel $xmltv_current_channel_name is not found in playlists\n";
                $xmltv_unknown_channel_set[$xmltv_current_channel_name] = 1;
            }
            $xmltv_current_channel_id = NULL;
            $xmltv_current_channel_name = NULL;
            $xmltv_current_channel_icon = NULL;
            break;
        case "PROGRAMME":
            if (isset($xmltv_channel_idset_map[$xmltv_current_channel_id]))
            {
                $programme_category_id = add_programme_category($xmltv_current_programme_category);
                $programme_id = add_programme($xmltv_current_programme_name, $xmltv_current_programme_description, $programme_category_id);
                $time_start = xmltv_parse_time($xmltv_current_programme_time_start) + $xmltv_time_offset;
                $time_stop = xmltv_parse_time($xmltv_current_programme_time_stop) + $xmltv_time_offset;
                foreach (array_keys($xmltv_channel_idset_map[$xmltv_current_channel_id]) as $channel_id)
                {
                    $timetable_id = add_timetable_programme($channel_id, $time_start, $time_stop, $programme_id);
                    $pdo->exec("insert into xmltv_timetable (xmltv_id, timetable_id) values($xmltv_id, $timetable_id)");
                    #echo "Timetable (id = $timetable_id) for programme $xmltv_current_programme_name",
                    #    " (category id = $programme_category_id, programme id = $programme_id",
                    #    ", channels ", join(",", array_keys($xmltv_channel_idset_map[$xmltv_current_channel_id])), ") added\n";
                    if (!isset($xmltv_channel_timetable_count[$channel_id]))
                        $xmltv_channel_timetable_count[$channel_id] = 1;
                    else
                        ++$xmltv_channel_timetable_count[$channel_id];
                }
            }
            else
            {
                #echo "Programme $xmltv_current_programme_name is for unknown channel - skipped\n";
                ++$xmltv_timetable_unknown_count;
            }
            $xmltv_current_channel_id = NULL;
            $xmltv_current_programme_time_start = NULL;
            $xmltv_current_programme_time_stop = NULL;
            $xmltv_current_programme_name = NULL;
            $xmltv_current_programme_description = NULL;
            $xmltv_current_programme_category = NULL;
            break;
        case "DISPLAY-NAME":
        case "TITLE":
        case "DESC":
        case "CATEGORY":
            $xmltv_state = update_xmltv_STATE_NONE;
            break;
    }
}

function xmltv_character_data_handler($parser, $data)
{
    global $xmltv_state, $xmltv_current_channel_name;
    global $xmltv_current_programme_name, $xmltv_current_programme_description, $xmltv_current_programme_category;

    switch ($xmltv_state)
    {
        case update_xmltv_STATE_CHANNEL_NAME:
            $xmltv_current_channel_name .= $data;
            break;
        case update_xmltv_STATE_PROGRAMME_NAME:
            $xmltv_current_programme_name .= $data;
            break;
        case update_xmltv_STATE_PROGRAMME_DESCRIPTION:
            $xmltv_current_programme_description .= $data;
            break;
        case update_xmltv_STATE_PROGRAMME_CATEGORY:
            $xmltv_current_programme_category .= $data;
            break;
    }
}

function parse_xmltv($name, $f, $time_offset)
{
    global $xmltv_state, $xmltv_id, $xmltv_channel_idset_map,
        $xmltv_current_channel_id, $xmltv_current_channel_name, $xmltv_current_channel_icon,
        $xmltv_current_programme_time_start, $xmltv_current_programme_time_stop, $xmltv_current_programme_name,
        $xmltv_current_programme_description, $xmltv_current_programme_category,
        $xmltv_unknown_channel_set, $xmltv_channel_timetable_count, $xmltv_timetable_unknown_count, $xmltv_channel_icons_count,
        $xmltv_time_offset;

    $xmltv_state = update_xmltv_STATE_NONE;
    $xmltv_id = NULL;
    $xmltv_channel_idset_map = array();
    $xmltv_current_channel_id = NULL;
    $xmltv_current_channel_icon = NULL;
    $xmltv_current_programme_time_start = NULL;
    $xmltv_current_programme_time_stop = NULL;
    $xmltv_current_programme_name = NULL;
    $xmltv_current_programme_description = NULL;
    $xmltv_current_programme_category = NULL;
    $xmltv_unknown_channel_set = array();
    $xmltv_channel_timetable_count = array();
    $xmltv_timetable_unknown_count = 0;
    $xmltv_channel_icons_count = 0;
    $xmltv_time_offset = $time_offset;

    try
    {
        $xml_parser = xml_parser_create("UTF-8");
        xml_set_element_handler($xml_parser, 'xmltv_start_element_handler', 'xmltv_end_element_handler');
        xml_set_character_data_handler($xml_parser, 'xmltv_character_data_handler');

        $xmltv_id = add_xmltv($name);
        echo "XMLTV id = $xmltv_id added\n";

        while($chunk = fread($f, 65536))
        {
            if (!xml_parse($xml_parser, $chunk))
            {
                echo update_xml_parser_errstr($xml_parser), "\n";
                return FALSE;
            }
        }
        if (!xml_parse($xml_parser, "", TRUE))
        {
            echo update_xml_parser_errstr($xml_parser), "\n";
            return FALSE;
        }

        echo "XMLTV listing $name:\n",
            array_sum(array_values($xmltv_channel_timetable_count)), " timetable entries for ", count(array_keys($xmltv_channel_timetable_count)), " channels\n",
            "$xmltv_channel_icons_count icons loaded\n",
            count(array_keys($xmltv_unknown_channel_set)), " not listed in playlists channels (", join(",", array_keys($xmltv_unknown_channel_set)), ")\n",
            "$xmltv_timetable_unknown_count timetable entries for not listed in playlists channels\n";
    }
    catch (Exception $e)
    {
        echo "Exception: ", $e->getMessage(), "\n";
        return FALSE;
    }
    return TRUE;
}

include_once "extproc_stream_filter.php";
include_once data_CONFIG;

$update_ok = 1;

putenv("PATH=../bin:" . getenv("PATH"));

if (file_exists(data_DB_NEW))
    unlink(data_DB_NEW);
$pdo = new PDO('sqlite:' . data_DB_NEW);
$pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
$pdo->beginTransaction();

init_channel_icon_dir();

/*
    Init data_DB
*/

$pdo->exec("create table channel_categories (id integer primary key autoincrement, name text not null)");
$pdo->exec("create table channels (id integer primary key autoincrement, name text not null, description text not null, url text not null, icon text not null, category_id integer not null)");
$pdo->exec("create table playlists (id integer primary key autoincrement, name text not null)");
$pdo->exec("create table playlist_channels (playlist_id integer not null, channel_id integer not null)");
$pdo->exec("create table programme_categories (id integer primary key autoincrement, name text not null)");
$pdo->exec("create table programmes (id integer primary key autoincrement, name text not null, description text not null, category_id integer not null)");
$pdo->exec("create table timetable (id integer primary key autoincrement, channel_id integer not null, time_start integer not null, time_stop integer not null, programme_id integer not null)");
$pdo->exec("create table xmltvs (id integer primary key autoincrement, name text not null)");
$pdo->exec("create table xmltv_timetable (xmltv_id integer not null, timetable_id integer not null)");

/*
    Import playlists
*/

foreach ($playlists as $playlist_name => $playlist_config)
{
    echo "\n";
    $f = fopen($playlist_config["url"], "r");
    if (!$f)
    {
        echo "Unable to open playlist $playlist_name (", $playlist_config["url"], ")\n";
        $update_ok = 0;
        continue;
    }

    switch (strtoupper($playlist_config["charset"]))
    {
        case "UTF8":
        case "UTF-8":
            break;
        default:
            if (!stream_filter_append($f, 'extproc.iconv -s -c -f ' . $playlist_config["charset"] . ' -t UTF-8'))
                exit("Failed to append filter extproc.iconv");
    }

    $result = 0;
    switch ($playlist_config["type"])
    {
        case "m3u":
            echo "Parsing m3u playlist $playlist_name\n";
            $result = parse_m3u($playlist_name, $f);
            break;
        case "xspf":
            echo "Parsing xspf playlist $playlist_name\n";
            $result = parse_xspf($playlist_name, $f);
            break;
        default:
            echo "Unknown playlist type: ", $playlist_config["type"], " ($playlist_name)\n";
    }

    if ($result)
        echo "Playlist $playlist_name parsed\n";
    else
    {
        echo "Playlist $playlist_name parsing FAILED\n";
        $update_ok = 0;
    }

    fclose($f);
}

/*
    Import XMLTV listings
*/

foreach ($xmltvs as $xmltv_name => $xmltv_config)
{
    echo "\n";
    $f = fopen($xmltv_config["url"], "r");
    if (!$f)
    {
        echo "Unable to open XMLTV listing $xmltv_name (", $xmltv_config["url"], ")\n";
        $update_ok = 0;
        continue;
    }

    switch ($xmltv_config["compression"])
    {
        case "gzip":
            if (!stream_filter_append($f, 'extproc.zcat'))
                exit("Failed to append filter extproc.zcat");
            break;
        case "bzip2":
            if (!stream_filter_append($f, 'extproc.bzcat'))
                exit("Failed to append filter extproc.bzcat");
            break;
        case "tar.bzip2":
            if (!stream_filter_append($f, 'extproc.tar -jxO -f -'))
                exit("Failed to append filter extproc.tar");
            break;
    }

    switch (strtoupper($xmltv_config["charset"]))
    {
        case "UTF8":
        case "UTF-8":
            break;
        default:
            if (!stream_filter_append($f, 'extproc.iconv -s -c -f ' . $xmltv_config["charset"] . ' -t UTF-8'))
                exit("Failed to append filter extproc.iconv");
            break;
    }

    $result = 0;
    echo "Parsing XMLTV listing $xmltv_name\n";

    $result = parse_xmltv($xmltv_name, $f, $xmltv_config["time_offset"] * 3600);

    if ($result)
        echo "XMLTV listing $xmltv_name parsed\n";
    else
    {
        echo "XMLTV listing $xmltv_name parsing FAILED\n";
        $update_ok = 0;
    }

    fclose($f);
}

if ($update_ok)
{
    /*
     * Init DB indexes
     */

    /*
     * scenario: view_all
     * channels(*) -> timetable(channel_id, time_start, time_stop) -> programmes(id)
     */
    $pdo->exec("create index timetable_idx_channel on timetable (channel_id, time_start, time_stop)");

    /*
     * scenario: view_channel_categories
     * channel_categories(*) -> channels(category_id) -> timetable(channel_id, time_start, time_stop) -> programmes(id)
     */
    $pdo->exec("create index channels_idx_category on channels (category_id)");

    /*
     * scenario: view_programme_categories
     * programme_categories(*) -> programmes(category_id) -> timetable(programme_id, time_start, time_stop) -> channels(id)
     */
    $pdo->exec("create index programmes_idx_category on programmes (category_id)");
    $pdo->exec("create index timetable_idx_programme on timetable (programme_id, time_start, time_stop)");

    /*
     * scenario: view_playlists
     * playlists(*) -> playlist_channels(playlist_id) -> channels(id) -> timetable(channel_id, time_start, time_stop) -> programmes(id)
     */
    $pdo->exec("create index playlist_channels_idx_playlist on playlist_channels (playlist_id)");

    /*
     * scenario: view_xmltv_listings
     * xmltvs(*) -> xmltv_timetable(xmltv_id) -> timetable(id, time_start, time_stop) -> programmes(id), channels(id)
     */
    $pdo->exec("create index xmltv_timetable_idx_xmltv on xmltv_timetable (xmltv_id)");
    $pdo->exec("create index timetable_idx_id on timetable (id, time_start, time_stop)");

    /*
     * scenario: programme.php
     * channels(id) -> channel_categories(id)
     * playlist_channels(channel_id) -> playlists(id)
     * timetable(channel_id, time_start, time_stop) -> programmes(id) -> programme_categories(id)
     * xmltv_timetable(timetable_id) -> xmltvs(id)
     */
    $pdo->exec("create index playlist_channels_idx_channel on playlist_channels (channel_id)");
    $pdo->exec("create index xmltv_timetable_idx_imetable on xmltv_timetable (timetable_id)");

    /*
     * Save data_DB
     */

    $pdo->commit();
    unset($pdo);
    unlink(data_DB);
    rename(data_DB_NEW, data_DB);

    commit_channel_icon_dir();
}
?>
