<?php

include_once "inc/config.php";

function save_php_config($fname)
{
    global $playlists, $xmltvs, $channel_aliases, $settings;

    if (file_exists($fname))
        unlink($fname);
    $f = fopen($fname, "w");
    if (!$f)
        exit("Unable to save php config. Please, reinstall to fix it.");

    fputs($f, "<?php\n");

    fputs($f, "\$playlists = array(\n");
    foreach ($playlists as $name => $config)
    {
        fputs($f, "    \"$name\" => array(\n");
        foreach ($config as $param_name => $param_value)
        {
            fputs($f, "        \"$param_name\" => \"$param_value\",\n");
        }
        fputs($f, "    ),\n");
    }
    fputs($f, ");\n");

    fputs($f, "\$xmltvs = array(\n");
    foreach ($xmltvs as $name => $config)
    {
        fputs($f, "    \"$name\" => array(\n");
        foreach ($config as $param_name => $param_value)
        {
            fputs($f, "        \"$param_name\" => \"$param_value\",\n");
        }
        fputs($f, "    ),\n");
    }
    fputs($f, ");\n");

    fputs($f, "\$channel_aliases = array(\n");
    foreach ($channel_aliases as $name => $names)
    {
        fputs($f, "    \"$name\" => array(\"" . join("\", \"", $names) . "\"),\n");
    }
    fputs($f, ");\n");

    fputs($f, "\$settings = array(\n");
    foreach ($settings as $name => $value)
    {
        fputs($f, "    \"$name\" => \"$value\",\n");
    }
    fputs($f, ");\n");

    if (!fputs($f, "?>\n"))
        exit("Unable to write php config. Please, reinstall to fix it.");

    fclose($f);
}

if (file_exists(data_CONFIG))
    include_once data_CONFIG;

define("update_check_NONE", 0);
define("update_check_FAILED", 1);
define("update_check_UPDATING", 2);
define("update_check_LKFAIL", 3);
define("update_check_UPDATED", 4);
$update_check_state = update_check_NONE;
$lock = @fopen(data_LOCK, "r");
if ($lock)
    if (flock($lock, LOCK_EX|LOCK_NB, $block) && file_exists(data_LOCK))
        $update_check_state = update_check_FAILED;
    else
        if ($block)
            $update_check_state = update_check_UPDATING;
        else
            $update_check_state = update_check_LKFAIL;
else
    $update_check_state = update_check_UPDATED;
unset($lock);

if (isset($_POST["update"]) || (isset($_GET["update_check"]) && $update_check_state == update_check_UPDATING))
    $meta_http_equiv = array("refresh" => "3; url=setup.php?update_check=1");

$title = "NMT CSI IPTV Setup";
include "inc/top.php";

$setup = 0;
if (isset($_POST["add_playlist"]))
{
    $playlists[$_POST["name"]] = array("type" => $_POST["type"], "url" => $_POST["url"], "charset" => $_POST["charset"]);
    save_php_config(data_CONFIG);
    echo "<hr><h1>Playlist ", $_POST["name"], " added</h1>";
    $setup = 1;
}

if (isset($_POST["delete_playlist"]))
{
    unset($playlists[$_POST["name"]]);
    save_php_config(data_CONFIG);
    echo "<hr><h1>Playlist ", $_POST["name"], " deleted</h1>";
    $setup = 1;
}

if (isset($_POST["add_xmltv"]))
{
    $xmltvs[$_POST["name"]] = array("url" => $_POST["url"], "charset" => $_POST["charset"], "compression" => $_POST["compression"], "time_offset" => $_POST["time_offset"]);
    save_php_config(data_CONFIG);
    echo "<hr><h1>XMLTV listing ", $_POST["name"], " added</h1>";
    $setup = 1;
}

if (isset($_POST["delete_xmltv"]))
{
    unset($xmltvs[$_POST["name"]]);
    save_php_config(data_CONFIG);
    echo "<hr><h1>XMLTV listing ", $_POST["name"], " deleted</h1>";
    $setup = 1;
}

if (isset($_POST["add_channel_alias"]))
{
    $channel_aliases[$_POST["name1"]][] = $_POST["name2"];
    save_php_config(data_CONFIG);
    echo "<hr><h1>Channel alias ", $_POST["name1"], " = ", $_POST["name2"], " added</h1>";
    $setup = 1;
}

if (isset($_POST["delete_channel_alias"]))
{
    unset($channel_aliases[$_POST["name"]]);
    save_php_config(data_CONFIG);
    echo "<hr><h1>Channel alias ", $_POST["name"], " deleted</h1>";
    $setup = 1;
}

include_once "inc/udpxy.php";

if (isset($_POST["set"]))
{
    $settings["udpxy"] = $_POST["udpxy"];
    $settings["update_period"] = $_POST["update_period"];
    $settings["proxy"] = $_POST["proxy"];
    $settings["prebuf"] = $_POST["prebuf"];
    $settings["mangling"] = $_POST["mangling"];
    $settings["fix_urls"] = $_POST["fix_urls"];
    save_php_config(data_CONFIG);
    echo "<hr><h1>Settings changed</h1>";
    echo "<pre>\n";
    $success = udpxy_restart();
    echo "</pre>\n";
    if ($success)
        echo "<h1>udpxy is started</h1>\n";
    $setup = 1;
}

if (isset($_POST["update"]))
{
    echo "<pre>\n";
    system("../update_now.sh");
    echo "</pre>\n";
    echo "<h1>Update started</h1>\n";
    echo "<a href=\"setup.php?update_check=1\">Check continuously</a><br>";
    $setup = 1;
}

if (isset($_POST["update_cleanup"]))
{
    if (unlink(data_LOCK))
        echo "<h1>Removed ", data_LOCK, "</h1>\n";
    $files = glob(data_LOG . "*");
    foreach ($files as $file)
        if (unlink($file))
            echo "<h1>Removed $file</h1>\n";
    $setup = 1;
}

if (isset($_GET["update_check"]))
{
    switch ($update_check_state)
    {
        case update_check_UPDATING:
            echo "<h3>Updating ...</h3>\n";
            echo "<a href=\"setup.php?update_check=1\">Check again</a><br>";
            break;
        case update_check_UPDATED:
            echo "<h1>Data updated</h1>\n";
            break;
        case update_check_LKFAIL:
            echo "<h2>Update check failed. Try again.</h2>\n";
            echo "<a href=\"setup.php?update_check=1\">Check again</a><br>";
            break;
        case update_check_FAILED:
            echo "<h2>Data update FAILED</h2>\n";
            break;
    }
    echo "<a href=\"", "viewer.php?url=" . urlencode(data_LOG), "\">update log</a><br>";
    $setup = 1;
}

if ($setup)
{
?>
<a href="setup.php">Setup</a><br>
<a href="index.php">Main screen</a><br>
<?php
}
else
{
?>
<a href="index.php">Main screen</a>

<hr>
<h2>Update data</h2>
<?php if ($update_check_state == update_check_UPDATING) { ?>
<h3>Updating ...</h3>
<a href="setup.php?update_check=1">Check continuously</a><br>
<?php } else if ($update_check_state == update_check_FAILED) { ?>
<form action="setup.php" method=post>
<input type=submit name=update_cleanup value="Cleanup failed update">
</form><br>
<?php } else { ?>
<form action="setup.php" method=post>
<input type=submit name=update value="Update">
</form><br>
<?php } ?>
<a href="<?php echo "viewer.php?url=" . urlencode(data_LOG); ?>">update log</a><br>
<a href="<?php echo "viewer.php?url=" . urlencode(data_LOG_old); ?>">old update log</a><br>

<hr>
<h2>Playlists</h2>
<table>
<tr>
    <th>name</th>
    <th>type</th>
    <th>url</th>
    <th>charset</th>
</tr>
<?php
foreach($playlists as $name => $config)
{
?>
<tr>
    <td style="color: green"><?php echo $name; ?></td>
    <td><?php echo $playlists[$name]["type"]; ?></td>
    <td><?php echo $playlists[$name]["url"]; ?></td>
    <td><?php echo $playlists[$name]["charset"]; ?></td>
    <td>
        <form action="setup.php" method=post>
            <input type=hidden name=name value="<?php echo $name; ?>">
            <input type=submit name=delete_playlist value="Delete">
        </form>
    </td>
</tr>
<?php
}
?>
</table>

<hr>
<h2>Add/Change playlist</h2>
<form action="setup.php" method=post>
<table>
<tr>
    <th align=right>name:</th>
    <td><input type=text name=name></td>
</tr>
<tr>
    <th align=right>type:</th>
    <td>
        <select name=type>
            <option value=m3u>m3u</option>
            <option value=xspf>xspf</option>
        </select>
    </td>
</tr>
<tr>
    <th align=right>url:</th>
    <td><input type=text name=url></td>
</tr>
<tr>
    <th align=right>charset:</th>
    <td><input type=text name=charset></td>
    <td>(UTF-8, CP1251, GB2312, ...)</td>
</tr>
<tr>
    <td><input type=submit name=add_playlist value="Add/Change"></td>
</tr>
</table>
</form>

<hr>
<h2>XMLTV listings</h2>
<table>
<tr>
    <th>name</th>
    <th>url</th>
    <th>charset</th>
    <th>compression</th>
    <th>time offset</th>
</tr>
<?php
foreach($xmltvs as $name => $config)
{
?>
<tr>
    <td style="color: green"><?php echo $name; ?></td>
    <td><?php echo $xmltvs[$name]["url"]; ?></td>
    <td><?php echo $xmltvs[$name]["charset"]; ?></td>
    <td><?php echo $xmltvs[$name]["compression"]; ?></td>
    <td><?php echo $xmltvs[$name]["time_offset"]; ?></td>
    <td>
        <form action="setup.php" method=post>
            <input type=hidden name=name value="<?php echo $name; ?>">
            <input type=submit name=delete_xmltv value="Delete">
        </form>
    </td>
</tr>
<?php
}
?>
</table>

<hr>
<h2>Add/Change XMLTV listing</h2>
<form action="setup.php" method=post>
<table>
<tr>
    <th align=right>name:</th>
    <td><input type=text name=name></td>
</tr>
<tr>
    <th align=right>url:</th>
    <td><input type=text name=url></td>
</tr>
<tr>
    <th align=right>charset:</th>
    <td><input type=text name=charset value="UTF-8"></td>
    <td>(UTF-8, CP1251, GB2312, ...)</td>
</tr>
<tr>
    <th align=right>compression:</th>
    <td>
        <select name=compression>
            <option value=none>none</option>
            <option value=gzip>gzip (.gz)</option>
            <option value=bzip2>bzip2 (.bz2)</option>
            <option value=tar.bzip2>tar.bzip2 (.tar.bz2 .tbz2 .tbz)</option>
        </select>
    </td>
</tr>
<tr>
    <th align=right>time offset:</th>
    <td><input type=text name=time_offset value="+0"></td>
    <td>(in hours: +1, +3, +8, -1, -8, -10, +1.5 ...)</td>
</tr>
<tr>
    <td><input type=submit name=add_xmltv value="Add/Change"></td>
</tr>
</table>
</form>

<hr>
<h2>Channel aliases</h2>
<table>
<tr>
<th>playlist</th><th>programme</th>
</tr>
<?php
foreach($channel_aliases as $name => $names)
{
?>
<tr>
    <td><?php echo $name; ?></td>
    <td><?php echo join(", ", $names); ?></td>
    <td>
        <form action="setup.php" method=post>
            <input type=hidden name=name value="<?php echo $name; ?>">
            <input type=submit name=delete_channel_alias value="Delete">
        </form>
    </td>
</tr>
<?php
}
?>
<form action="setup.php" method=post>
<tr>
    <td>
	    <input type=text name=name1>
    </td>
    <td>
	    <input type=text name=name2>
    </td>
    <td>
        <input type=submit name=add_channel_alias value="Add">
    </td>
</tr>
</form>
</table>

<hr>
<h2>Settings</h2>
<?php
function print_options($current_value, $options)
{
    foreach ($options as $value => $name)
    {
        if ($value == $current_value)
            echo "<option value=$value selected>$name</option>";
        else
            echo "<option value=$value>$name</option>";
    }
}
?>
<form action="setup.php" method=post>
<table>
<tr>
    <th align=right nowrap>Fix urls:</th>
    <td>
        <select name=fix_urls>
            <?php print_options($settings["fix_urls"], array(
                            0 => "Disabled",
                            1 => "Enabled"
                            )); ?>
        </select>
    </td>
    <td>
    	(
    	some firmware does not support rtp:// url scheme,
    	so it should be fixed to udp://
    	Enable it if you don't use any proxy.
    	)
    </td>
</tr>
<tr>
    <th align=right nowrap>RTP/UDP proxy:</th>
    <td><input type=text name=udpxy value="<?php echo $settings["udpxy"]; ?>"></td>
    <td>
    	(
    	line &quot;localhost:14022&quot; or IP that starts with 127 will start &amp; use local proxy,
    	other host will not start local proxy but will be used as proxy,
    	empty line will not start local proxy &amp; no proxy will be used,
    	reboot may be needed or try another port in case local proxy is not starting
    	)
    </td>
</tr>
<tr>
	<td><a href="<?php echo "viewer.php?url=" . urlencode(data_UDPXY_LOG); ?>">udpxy log</a></td>
	<td>
		<?php
		    if (udpxy_check_started())
		        echo "udpxy is started";
		?>
	</td>
	<td>
		<?php
		    if (udpxy_check_running())
		        echo "udpxy is running";
		?>
	</td>
</tr>
<tr>
    <th align=right nowrap>Additional proxy:</th>
    <td>
        <select name=proxy>
            <?php print_options($settings["proxy"], array(
                            0 => "Disabled",
                            1 => "Enabled"
                            )); ?>
        </select>
    </td>
    <td>
    	(
    	video stream may be proxied by additional proxy-script written on php,
    	which represents the video stream like it would have been a bare video file.
    	This trick may be used for some firmware, that sends extra HTTP HEAD requests,
    	which udpxy is not able to handle. But it may slowdown things and cause interrupts.
    	)
    </td>
</tr>
<tr>
    <th align=right nowrap>Update period:</th>
    <td>
        <select name=update_period>
            <?php print_options($settings["update_period"], array(
                            0 => "Disabled",
                            10800 => "3 Hours",
                            21600 => "6 Hours",
                            43200 => "12 Hours",
                            86400 => "1 Day",
                            259200 => "3 Days",
                            604800 => "1 Week",
                            1209600 => "2 Weeks",
                            2419200 => "4 Weeks"
                            )); ?>
        </select>
    </td>
</tr>
<tr>
    <th align=right nowrap>Prebuffering:</th>
    <td>
        <select name=prebuf>
            <?php print_options($settings["prebuf"], array(
                            0 => "Default",
                            4096 => "4 kb",
                            16384 => "16 kb",
                            65536 => "64 kb",
                            262144 => "256 kb",
                            524288 => "512 kb",
                            1048576 => "1 Mb"
                            )); ?>
        </select>
    </td>
</tr>
<tr>
    <th align=right nowrap>Channel name mangling:</th>
    <td>
        <select name=mangling>
            <?php print_options($settings["mangling"], array(
                            0 => "Disabled",
                            1 => "Enabled"
                            )); ?>
        </select>
    </td>
    <td>
    	(
    	try to guess channel aliases using standard name mangling
    	)
    </td>
</tr>
<tr>
    <td><input type=submit name=set value="Set"></td>
</tr>
</table>
</form>

<?php
}
include "inc/bottom.php";
?>
